using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;

namespace Registry_CS
{
    class Registry
    {
        /// <summary>
        ///  
        /// </summary>
        /// <param name="keyName">  </param>
        /// <returns>    ERROR_SUCCESS</returns>
        public static int CreateKey(UIntPtr root, string keyName)
        {
            UIntPtr hkey = UIntPtr.Zero;
            uint disposition = 0;

            try
            {
                return RegCreateKeyEx(root, keyName, 0, null, 0,
                    KeyAccess.None, IntPtr.Zero, ref hkey, ref disposition);
            }
            finally
            {
                if (UIntPtr.Zero != hkey)
                {
                    RegCloseKey(hkey);
                }
            }
        }

        /// <summary>
        ///  
        /// </summary>
        /// <param name="keyName"> </param>
        /// <returns>    ERROR_SUCCESS</returns>
        public static int DeleteKey(UIntPtr root, string keyName)
        {
            return RegDeleteKey(root, keyName);
        }

        /// <summary>
        ///      
        /// </summary>
        /// <param name="keyName"> </param>
        /// <param name="valueName"> </param>
        /// <param name="stringData"> </param>
        /// <returns>    ERROR_SUCCESS</returns>
        public static int CreateValueString(UIntPtr root, string keyName, string valueName, string stringData)
        {
            UIntPtr hkey = UIntPtr.Zero;

            try
            {
                int result = RegOpenKeyEx(root, keyName, 0, KeyAccess.None, ref hkey);
                if (ERROR_SUCCESS != result)
                    return result;

                byte[] bytes = Encoding.Unicode.GetBytes(stringData);

                return RegSetValueEx(hkey, valueName, 0, KeyType.String,
                    bytes, (uint)bytes.Length);
            }
            finally
            {
                if (UIntPtr.Zero != hkey)
                {
                    RegCloseKey(hkey);
                }
            }
        }

        /// <summary>
        ///    DWORD   
        /// </summary>
        /// <param name="keyName"> </param>
        /// <param name="valueName"> </param>
        /// <param name="dwordData"> </param>
        /// <returns>    ERROR_SUCCESS</returns>
        public static int CreateValueDWORD(UIntPtr root, string keyName, string valueName, uint dwordData)
        {
            UIntPtr hkey = UIntPtr.Zero;

            try
            {
                int result = RegOpenKeyEx(root, keyName, 0, KeyAccess.None, ref hkey);
                if (ERROR_SUCCESS != result)
                    return result;

                byte[] bytes = BitConverter.GetBytes(dwordData);
                return RegSetValueEx(hkey, valueName, 0, KeyType.Dword,
                    bytes, (uint)bytes.Length);
            }
            finally
            {
                if (UIntPtr.Zero != hkey)
                {
                    RegCloseKey(hkey);
                }
            }
        }

        /// <summary>
        ///      
        /// </summary>
        /// <param name="keyName"> </param>
        /// <param name="valueName"> </param>
        /// <param name="dwordData"> </param>
        /// <returns>    ERROR_SUCCESS</returns>
        public static int CreateValueBinary(UIntPtr root, string keyName, string valueName, uint binData)
        {
            UIntPtr hkey = UIntPtr.Zero;

            try
            {
                int result = RegOpenKeyEx(root, keyName, 0, KeyAccess.None, ref hkey);
                if (ERROR_SUCCESS != result)
                    return result;
                               
                byte[] data = BitConverter.GetBytes(binData);
                return RegSetValueEx(hkey, valueName, 0, KeyType.Binary,
                    data, (uint)data.Length);
            }
            finally
            {
                if (UIntPtr.Zero != hkey)
                {
                    RegCloseKey(hkey);
                }
            }
        }


        /// <summary>
        ///    
        /// </summary>
        /// <param name="keyName"> </param>
        /// <param name="valueName"> </param>
        /// <param name="stringResult"> </param>
        /// <returns>    ERROR_SUCCESS</returns>
        public static int GetStringValue(UIntPtr root, string keyName, string valueName, ref string stringResult)
        {
            UIntPtr hkey = UIntPtr.Zero;

            try
            {
                int result = RegOpenKeyEx(root, keyName, 0, KeyAccess.None, ref hkey);
                if (ERROR_SUCCESS != result)
                    return result;

                byte[] bytes = null;
                uint length = 0;
                KeyType keyType = KeyType.None;

                result = RegQueryValueEx(hkey, valueName, IntPtr.Zero, ref keyType,
                    null, ref length);

                if (ERROR_SUCCESS != result)
                    return result;

                keyType = KeyType.None;
                bytes = new byte[length];

                result = RegQueryValueEx(hkey, valueName, IntPtr.Zero, ref keyType,
                    bytes, ref length);

                if (ERROR_SUCCESS != result)
                    return result;

                stringResult = Encoding.Unicode.GetString(bytes, 0, bytes.Length);

                return ERROR_SUCCESS;
            }
            finally
            {
                if (UIntPtr.Zero != hkey)
                {
                    RegCloseKey(hkey);
                }
            }
        }

        /// <summary>
        ///     DWORD
        /// </summary>
        /// <param name="keyName"> </param>
        /// <param name="valueName"> </param>
        /// <param name="dwordResult"> </param>
        /// <returns>    ERROR_SUCCESS</returns>
        public static int GetDWORDValue(UIntPtr root, string keyName, string valueName, ref uint dwordResult)
        {
            UIntPtr hkey = UIntPtr.Zero;

            try
            {
                int result = RegOpenKeyEx(root, keyName, 0, KeyAccess.None, ref hkey);
                if (ERROR_SUCCESS != result)
                    return result;

                byte[] bytes = null;
                uint length = 0;
                KeyType keyType = KeyType.None;

                result = RegQueryValueEx(hkey, valueName, IntPtr.Zero, ref keyType,
                    null, ref length);

                bytes = new byte[Marshal.SizeOf(typeof(uint))];
                length = (uint)bytes.Length;
                keyType = KeyType.None;

                result = RegQueryValueEx(hkey, valueName, IntPtr.Zero, ref keyType,
                    bytes, ref length);

                if (ERROR_SUCCESS != result)
                    return result;

                dwordResult = BitConverter.ToUInt32(bytes, 0);

                return ERROR_SUCCESS;
            }
            finally
            {
                if (UIntPtr.Zero != hkey)
                {
                    RegCloseKey(hkey);
                }
            }
        }

        /// <summary>
        ///      
        /// </summary>
        /// <param name="keyName"> </param>
        /// <param name="valueName"> </param>
        /// <returns>    ERROR_SUCCESS</returns>
        public static int DeleteValue(UIntPtr root, string keyName, string valueName)
        {
            UIntPtr hkey = UIntPtr.Zero;

            try
            {
                int result = RegOpenKeyEx(root, keyName, 0, KeyAccess.None, ref hkey);
                if (ERROR_SUCCESS != result)
                    return result;

                return RegDeleteValue(hkey, valueName);
            }
            finally
            {
                if (UIntPtr.Zero != hkey)
                {
                    RegCloseKey(hkey);
                }
            }
        }

        /// <summary>
        ///  
        /// </summary>
        public enum KeyType : uint
        {
            None = 0,
            String = 1,
            Binary = 3,
            Dword = 4,
        }

        /// <summary>
        ///  
        /// </summary>
        public enum KeyAccess : uint
        {
            None = 0x0000,
            QueryValue = 0x0001,
            SetValue = 0x0002,
            CreateSubKey = 0x0004,
            EnumerateSubKeys = 0x0008,
            Notify = 0x0010,
            CreateLink = 0x0020
        }

        /// <summary>
        /// HKEY_CLASSES_ROOT
        /// </summary>
        public static UIntPtr HKCR = new UIntPtr(0x80000000);
        /// <summary>
        /// HKEY_CURRENT_USER
        /// </summary>
        public static UIntPtr HKCU = new UIntPtr(0x80000001);
        /// <summary>
        /// HKEY_LOCAL_MACHINE
        /// </summary>
        public static UIntPtr HKLM = new UIntPtr(0x80000002);

        /// <summary>
        /// HKEY_USERS
        /// </summary>
        public static UIntPtr HKU = new UIntPtr(0x80000003);

        /// <summary>
        ///     
        /// </summary>
        public const int ERROR_SUCCESS = 0;

        /// <summary>
        ///      .    ,
        ///    .
        /// </summary>
        /// <param name="hkey">[in]         :
        /// HKCR, HKCU, HKLM.</param>
        /// <param name="lpSubKey">[in]    .     
        /// ,    hKey.
        /// </param>
        /// <param name="Reserved">[in]  .   0</param>
        /// <param name="lpClass">[in]     
        ///   ,    
        ///</param>
        /// <param name="dwOptions">[in] ;   0
        /// </param>
        /// <param name="samDesired">[in] ;   0
        /// </param>
        /// <param name="lpSecurityAttributes">[in]   NULL.
        /// </param>
        /// <param name="phkResult">[out] ,   
        ///    
        ///       ,  
        ///  RegCloseKey   . </param>
        /// <param name="lpdwDisposition">[out] ,    1 (REG_CREATED_NEW_KEY),
        ///    
        ///   2 (REG_OPENED_EXISTING_KEY),      
        /// </param>
        /// <returns>ERROR_SUCCESS     .    
        ///    
        /// </returns>
        [DllImport("coredll.dll", SetLastError = true)]
        public static extern int RegCreateKeyEx
        (
            UIntPtr hkey,
            String lpSubKey,
            uint Reserved,
            StringBuilder lpClass,
            uint dwOptions,
            KeyAccess samDesired,
            IntPtr lpSecurityAttributes,
            ref UIntPtr phkResult,
            ref uint lpdwDisposition
        );

        /// <summary>
        ///      
        /// </summary>
        /// <param name="hkey">[in]         :
        /// HKCR, HKCU, HKLM.
        /// </param>
        /// <param name="subkeyName">[in]   .   NULL
        /// </param>
        /// <returns>ERROR_SUCCESS     .    
        ///   
        /// </returns>
        [DllImport("coredll.dll", SetLastError = true)]
        public static extern int RegDeleteKey
        (
            UIntPtr hkey,
            string subkeyName
        );

        /// <summary>
        ///      .
        /// </summary>
        /// <param name="hkey">[in]         :
        /// HKCR, HKCU, HKLM.</param>
        /// <param name="lpSubKey">[in]   
        /// </param>
        /// <param name="ulOptions">[in]  .   0</param>
        /// <param name="samDesired">[in]  .   0.</param>
        /// <param name="phkResult">[out] ,     
        ///       , 
        ///     RegCloseKey   </param>
        /// <returns>ERROR_SUCCESS     .    
        ///   
        /// </returns>
        [DllImport("coredll.dll", SetLastError = true)]
        public static extern int RegOpenKeyEx
        (
            UIntPtr hkey,
            String lpSubKey,
            uint ulOptions,
            KeyAccess samDesired,
            ref UIntPtr phkResult
        );

        /// <summary>
        ///         
        /// </summary>
        /// <param name="hkey">[in]         :
        /// HKCR, HKCU, HKLM.</param>
        /// <param name="lpValueName">[in]  . 
        /// </param>
        /// <param name="lpReserved">[in]  .   NULL.</param>
        /// <param name="lpType">[out]  P
        /// </param>
        /// <param name="lpData">[out] ,  .
        ///     NULL,    .
        /// </param>
        /// <param name="lpcbData">[in/out]     
        ///</param>
        /// <returns>ERROR_SUCCESS     .    
        ///   
        /// </returns>
        [DllImport("coredll.dll", SetLastError = true)]
        public static extern int RegQueryValueEx
        (
            UIntPtr hkey,
            String lpValueName,
            IntPtr lpReserved,
            ref KeyType lpType,
            byte[] lpData,
            ref uint lpcbData
        );

        /// <summary>
        ///      .
        /// </summary>
        /// <param name="hkey">[in] Handle to a currently open key or one of:
        /// HKCR, HKCU, HKLM.</param>
        /// <param name="lpValueName">[in] Pointer to a string containing the
        /// name of the value to set. If a value with this name is not already
        /// present in the key, the function adds it to the key. If this parameter
        /// is NULL or an empty string, the function sets the type and data for the
        /// keys unnamed value. Registry keys do not have default values, but they
        /// can have one unnamed value, which can be of any type. The maximum length
        /// of a value name is 255, not including the terminating NULL
        /// character. </param>
        /// <param name="Reserved">[in] Reserved; must be zero.</param>
        /// <param name="dwType">[in] Specifies the type of information to be stored
        /// as the values data.</param>
        /// <param name="lpData">[in] Pointer to a buffer containing the data to
        /// be stored with the specified value name.</param>
        /// <param name="cbData">[in] Specifies the size, in bytes, of the
        /// information pointed to by the lpData parameter. If the data is of
        /// type REG_SZ, REG_EXPAND_SZ, or REG_MULTI_SZ, cbData must include the
        /// size of the terminating null character. The maximum size of data allowed
        /// in Windows CE is 4 KB.</param>
        /// <returns>ERROR_SUCCESS indicates success. A nonzero error code defined
        /// in Winerror.h indicates failure. To get a generic description of the
        /// error, call FormatMessage with the FORMAT_MESSAGE_FROM_SYSTEM flag set.
        /// The message resource is optional; therefore, if you call FormatMessage
        /// it could fail.</returns>
        [DllImport("coredll.dll", SetLastError = true)]
        public static extern int RegSetValueEx
        (
            UIntPtr hkey,
            String lpValueName,
            uint Reserved,
            KeyType dwType,
            byte[] lpData,
            uint cbData
        );

        /// <summary>
        /// This function removes a named value from the specified registry key.
        /// A remote application interface (RAPI) version of this function exists,
        /// and it is called CeRegDeleteValue. 
        /// </summary>
        /// <param name="hkey">[in] Handle to a currently open key or one of:
        /// HKCR, HKCU, HKLM.</param>
        /// <param name="valueName">[in] Pointer to a null-terminated string
        /// that names the value to remove. If this parameter is NULL or points
        /// to an empty string, the default value of the key is removed. A default
        /// value is create by calling RegSetValueEx with a NULL or empty string
        /// value name.</param>
        /// <returns>ERROR_SUCCESS indicates success. A nonzero error code defined
        /// in Winerror.h indicates failure. To get a generic description of the
        /// error, call FormatMessage with the FORMAT_MESSAGE_FROM_SYSTEM flag set.
        /// The message resource is optional; therefore, if you call FormatMessage
        /// it could fail.</returns>
        [DllImport("coredll.dll", SetLastError = true)]
        public static extern int RegDeleteValue
        (
            UIntPtr hkey,
            string valueName
        );

        /// <summary>
        /// This function releases the handle of the specified key. A remote
        /// application interface (RAPI) version of this function exists, and
        /// it is called CeRegCloseKey.
        /// </summary>
        /// <param name="hkey">[in] Handle to the open key to close.</param>
        /// <returns>ERROR_SUCCESS indicates success. A nonzero error code
        /// defined in Winerror.h indicates failure. To get a generic description
        /// of the error, call FormatMessage with the FORMAT_MESSAGE_FROM_SYSTEM
        /// flag set. The message resource is optional; therefore, if you call
        /// FormatMessage it could fail.</returns>
        [DllImport("coredll.dll", SetLastError = true)]
        public static extern int RegCloseKey
        (
            UIntPtr hkey
        );

        /// <summary>
        /// Run a test of the Registry class.
        /// </summary>
        /// <param name="showLine">Delegate called to show debug information</param>
        //public static void TestProc(MainTest.DisplayLineDelegate showLine)
        //{
        //    string keyName = @"MyKey";
        //    string stringValueName = @"MyStringValue";
        //    string DWORDValueName = @"MyDwordValue";
        //    string stringData = @"MyStringData";
        //    uint DWORDData = 0xABCDEF01;
        //    uint resultDWORDData = 0x00000000;
        //    string resultStringData = "Empty";

        //    showLine(string.Format("Creating key {0}", keyName));
        //    int result = CreateKey(keyName);
        //    if (result != Registry.ERROR_SUCCESS)
        //    {
        //        showLine("FAILURE: Failed to create key");
        //        return;
        //    }

        //    showLine(string.Format("Creating DWORD value 0x{0:X8}", DWORDData));
        //    result = CreateValueDWORD(keyName, DWORDValueName, DWORDData);
        //    if (result != Registry.ERROR_SUCCESS)
        //        showLine("FAILURE: Failed to create value");

        //    showLine(string.Format("Creating string value {0}", stringData));
        //    result = CreateValueString(keyName, stringValueName, stringData);
        //    if (result != Registry.ERROR_SUCCESS)
        //        showLine("FAILURE: Failed to create value");

        //    showLine("Getting DWORD value");
        //    result = GetDWORDValue(keyName, DWORDValueName, ref resultDWORDData);
        //    if (result != Registry.ERROR_SUCCESS)
        //    {
        //        showLine("FAILURE: Failed to get value");
        //    }
        //    else
        //    {
        //        if (resultDWORDData == DWORDData)
        //            showLine(string.Format("Returned proper value: 0x{0:X8}", DWORDData));
        //        else
        //            showLine(string.Format("FAILURE: Set:0x{0:X8} Get:0x{1:X8}", DWORDData, resultDWORDData));
        //    }

        //    showLine("Getting string value");
        //    result = GetStringValue(keyName, stringValueName, ref resultStringData);
        //    if (result != Registry.ERROR_SUCCESS)
        //    {
        //        showLine("FAILURE: Failed to get value");
        //    }
        //    else
        //    {
        //        if (resultStringData == stringData)
        //            showLine(string.Format("Returned proper value: {0}", stringData));
        //        else
        //            showLine(string.Format("FAILURE: Set:{0} Get:{1}", stringData, resultStringData));
        //    }

        //    showLine("Deleting DWORD value");
        //    result = DeleteValue(keyName, DWORDValueName);
        //    if (result != Registry.ERROR_SUCCESS)
        //        showLine("FAILURE: Failed to delete value");

        //    showLine("Deleting string value");
        //    result = DeleteValue(keyName, stringValueName);
        //    if (result != Registry.ERROR_SUCCESS)
        //        showLine("FAILURE: Failed to delete value");

        //    showLine("Deleting key");
        //    result = DeleteKey(keyName);
        //    if (result != Registry.ERROR_SUCCESS)
        //        showLine("FAILURE: Failed to delete key");

        //}

        //public void SetValue(string name, object value, RegistryValueKind valueKind)
        //{
        //    if (m_writable)
        //    {
        //        if (CheckHKey())
        //        {
        //            byte[] data;

        //            switch (valueKind)
        //            {
        //                case RegistryValueKind.String:
        //                    data = System.Text.Encoding.Unicode.GetBytes((string)value + '\0');
        //                    break;
        //                case RegistryValueKind.MultiString:
        //                    System.Text.StringBuilder sb = new System.Text.StringBuilder();

        //                    foreach (string str in (string[])value)
        //                        sb.Append(str + '\0');
        //                    sb.Append('\0'); // terminated by two null characters
        //                    data = System.Text.Encoding.Unicode.GetBytes(sb.ToString());
        //                    break;
        //                case RegistryValueKind.Binary:
        //                    data = (byte[])value;
        //                    break;
        //                case RegistryValueKind.DWord:
        //                    if (value is UInt32)
        //                    {
        //                        data = BitConverter.GetBytes((uint)value);
        //                    }
        //                    else
        //                    {
        //                        data = BitConverter.GetBytes(Convert.ToInt32(value));
        //                    }
        //                    break;

        //                default:
        //                    SetValue(name, value);
        //                    return;
        //            }

        //            int size = data.Length;
        //            int result = RegSetValueEx(m_handle, name, 0, valueKind, data, size);


        //            if (result != 0)
        //            {
        //                throw new Win32Exception(Marshal.GetLastWin32Error(), "Error writing to the RegistryKey");
        //            }
        //        }
        //        else
        //        {
        //            throw new ObjectDisposedException("The RegistryKey being manipulated is closed (closed keys cannot be accessed).");
        //        }
        //    }
        //    else
        //    {
        //        throw new UnauthorizedAccessException("Cannot set value on RegistryKey which was opened as ReadOnly");
        //    }
        //}
    }
}
